use Test::More tests => 4;

use_ok( "Net::Async::Tangence::Protocol" );
use_ok( "Net::Async::Tangence::ServerProtocol" );
use_ok( "Net::Async::Tangence::Client" );
use_ok( "Net::Async::Tangence::Server" );
